<?php

namespace Drupal\webt\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\webt\translation_engines\etranslation\EtranslationService;
use Drupal\webt\translation_engines\etranslation\EtranslationUtils;
use Drupal\webt\WebtConstInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller that handles eTranslation async responses.
 */
class EtranslationCallbackController extends ControllerBase {

  use LoggerChannelTrait;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The translation manager service.
   *
   * @var \Drupal\webt\TranslationManager
   */
  protected $translationManager;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new class instance.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache service.
   * @param \Drupal\webt\TranslationManager $translationManager
   *   The translation manager service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct($requestStack, $cache, $translationManager, $languageManager, $entityTypeManager) {
    $this->requestStack = $requestStack;
    $this->cache = $cache;
    $this->translationManager = $translationManager;
    $this->languageManager = $languageManager;
    $this->logger = $this->getLogger(WebtConstInterface::WEBT_LOGGER);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal service container.
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('cache.default'),
      $container->get('webt.translation_manager'),
      $container->get('language_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Receive & process async translation response from eTranslation.
   *
   * If this is a translaton for an entity and timeout has been reached before,
   * also parse and save translation.
   *
   * @return Symfony\Component\HttpFoundation\Response
   *   Response
   */
  public function onResponse() {
    $request            = $this->requestStack->getCurrentRequest();
    $request_id         = Html::escape($request->get('request-id'));
    $translated_content = base64_decode(file_get_contents('php://input'));
    $key                = EtranslationUtils::getCacheKey($request_id);
    $this->logger->info("Received response from eTranslation ($request_id)");
    if ($this->cache->get($key) && $this->cache->get($key)->data === EtranslationUtils::$timeoutValue) {
      $this->logger->info("Received late translation ($request_id)");
      $entity_id = Html::escape($request->get('external-reference'));
      if ($entity_id) {
        $entity = $this->loadEntityById($entity_id);
        if ($entity) {
          $this->processEntityTranslation($entity, $translated_content, Html::escape($request->get('target-language')));
        }
      }
    }
    else {
      $this->cache->set($key, $translated_content);
    }
    return new Response();
  }

  /**
   * Process async error response from eTranslation.
   *
   * Logs error and marks translation
   * as completed with error.
   *
   * @return Symfony\Component\HttpFoundation\Response
   *   Response
   */
  public function onError() {
    $request    = $this->requestStack->getCurrentRequest();
    $request_id = Html::escape($request->get('request-id'));
    $this->logger->error("eTranslation response error ($request_id). Please make sure eTranslation API is working as expected.");
    $this->cache->set(EtranslationUtils::getCacheKey($request_id), EtranslationUtils::$errorValue);
    return new Response();
  }

  /**
   * Parse translation response and insert entity translation.
   *
   * @param [type] $entity
   *   Original entity to translate.
   * @param [type] $translated_content
   *   Translation received from eTranslation service.
   * @param [type] $target_language
   *   Language of translation received.
   */
  private function processEntityTranslation($entity, $translated_content, $target_language) {
    $source_langcode     = $entity->language()->getId();
    $target_langcode     = $this->getLangcodeByLanguage($target_language);
    $translation_service = $this->translationManager->translationService;
    if ($translation_service instanceof EtranslationService) {
      $xml_translations = $translation_service->etranslationResponseToTranslationArray($translated_content);

      $translatable_fields = $this->translationManager->getTranslatableFields($entity);
      $original_values     = $this->translationManager->extractTranslatableValues($entity, $translatable_fields);
      if (count($original_values) === count($xml_translations)) {
        $translations = $translation_service->decodeXmlTranslations($original_values, $xml_translations);
        $this->translationManager->translateEntity($entity, $source_langcode, $target_langcode, $translations);
      }
    }
  }

  /**
   * Retrieve entity by ID.
   *
   * @param [type] $id
   *   Identifier of entity.
   *
   * @return \Drupal\Core\Entity\Entity
   *   Entity or null if enitity was not found
   */
  private function loadEntityById($id) {
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if ($entity_type->entityClassImplements('\Drupal\Core\Entity\EntityInterface')) {
        $storage = $this->entityTypeManager->getStorage($entity_type_id);
        $entity  = $storage->load($id);
        if ($entity) {
          return $entity;
        }
      }
    }
    $this->logger->error("Could not load entity by ID ($id). Please make sure such entity exists!");
    return NULL;
  }

  /**
   * In languages added to the website find langcode for given language.
   *
   * @param string $language
   *   Requested language e.g. 'en'.
   *
   * @return string
   *   Langcode or null if langcode not found.
   */
  private function getLangcodeByLanguage($language) {
    $languages = $this->languageManager->getLanguages();
    $language_lowercase = strtolower($language);

    foreach ($languages as $langcode => $value) {
      if (str_starts_with($langcode, $language_lowercase)) {
        return $langcode;
      }
    }
    $this->logger->error("Could not find langcode for language '$language_lowercase'. Please check if the language code of this language is correct!");
    return NULL;
  }

}
